//+------------------------------------------------------------------+
//|                                                    CSignalMC.mqh |
//|                   Copyright 2009-2013, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <Math\Alglib\dataanalysis.mqh>
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals of indicator 'Markov Chain for Population Data'    |
//| Type=SignalAdvanced                                              |
//| Name=MarkovChain                                                 |
//| ShortName=MarkovChain                                            |
//| Class=CSignalMC                                                  |
//| Page=signal_mc                                                   |
//| Parameter=SignalTracks,int,5,Signal Data Points                  |
//| Parameter=SignalRegulizer,double,0.05,Signal Regulizer           |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CSignalMC.                                                 |
//| Purpose: Class of generator of trade signals based on            |
//|          the 'Markov Chains for Population Data' indicator.      |
//| Is derived from the CExpertSignal class.                         |
//+------------------------------------------------------------------+
#define     __S_STATES 5
//
#define		__S_PHI (sqrt(5.0)*0.5)-0.5
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CSignalMC : public CExpertSignal
  {
protected:
   CMarkovCPD        CPD;                    // object-indicator
   
   //--- adjusted parameters
   int               m_signal_tracks;        // the data tracks
   double            m_signal_regulizer;     // the neutrality definer 

public:
   
                     CSignalMC(void);
                    ~CSignalMC(void);
   //--- methods of setting adjustable parameters
   void              SignalTracks(int value)             { m_signal_tracks=value;         }
   void              SignalRegulizer(double value)       { m_signal_regulizer=value;      }
   //--- method of verification of settings
   virtual bool      ValidationSettings(void);
   //--- method of creating the indicator and timeseries
   virtual bool      InitIndicators(CIndicators *indicators);
   //--- methods of checking if the market models are formed
   virtual int       LongCondition(void);
   virtual int       ShortCondition(void);

protected:
   
   double            ProcessMC(int Index);
   double            GetState(double NewChange,double OldChange);
  };
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CSignalMC::CSignalMC(void) : m_signal_tracks(5),m_signal_regulizer(0.05)
  {
//--- initialization of protected data
   m_used_series=USE_SERIES_CLOSE;
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CSignalMC::~CSignalMC(void)
  {
  }
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//+------------------------------------------------------------------+
bool CSignalMC::ValidationSettings(void)
  {
//--- validation settings of additional filters
   if(!CExpertSignal::ValidationSettings())
      return(false);
//--- initial data checks
   if(m_signal_tracks<3)
     {
      printf(__FUNCTION__+": signal tracks must be greater than 2");
      return(false);
     }
   if(m_signal_regulizer<=0.0)
     {
      printf(__FUNCTION__+": signal regulizer must be greater than 0.0");
      return(false);
     }
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//+------------------------------------------------------------------+
bool CSignalMC::InitIndicators(CIndicators *indicators)
  {
//--- check pointer
   if(indicators==NULL)
      return(false);
//--- initialization of indicators and timeseries of additional filters

//--- 
   if(!CExpertSignal::InitIndicators(indicators))
      return(false);
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Markov Chain method                                              |
//| INPUT PARAMETERS                                                 |
//|     Index   -   int, read index within price buffer.             |
//| OUTPUT                                                           |
//|     double  -   Normalized value (-1.0 to +1.0) for signal       |
//+------------------------------------------------------------------+
double CSignalMC::ProcessMC(int Index)
   {
      double _mc=0.0;
      
      CMCPDState _s;
      CMatrixDouble _xy,_p;
      CMCPDReport _rep;
      int _k=m_signal_tracks;
      
      _xy.Resize(m_signal_tracks,__S_STATES);
      
      m_close.Refresh(-1);
      
      for(int t=0;t<m_signal_tracks;t++)
      {
         for(int s=0;s<__S_STATES;s++)
         {
            _xy[t].Set(s,GetState(Close(Index+t+s)-Close(Index+t+s+1),Close(Index+t+s+1)-Close(Index+t+s+2)));
         }
      }
      
      CPD.MCPDCreate(__S_STATES,_s);
      
      CPD.MCPDAddTrack(_s,_xy,_k);
      
      CPD.MCPDSetTikhonovRegularizer(_s,m_signal_regulizer);
      
      CPD.MCPDSolve(_s);
      
      CPD.MCPDResults(_s,_p,_rep);
      
      _mc=_xy[0][0];
      double _probability=_p[__S_STATES-1][0];
      //
      for(int s=0;s<__S_STATES;s++)
      {
         if(_probability<_p[__S_STATES-1][s])
         {
            _mc=_xy[0][s];
            _probability=_p[__S_STATES-1][s];
         }
      }
      
      _mc-=1.0;
      
      return(_mc);
   }
//+------------------------------------------------------------------+
//| Normalizer.                                                      |
//+------------------------------------------------------------------+
double CSignalMC::GetState(double NewChange,double OldChange)
   {
      double _state=0.0;
      
      double _norm=fabs(NewChange)/fmax(m_symbol.Point(),fabs(NewChange)+fabs(OldChange));
      
      if(NewChange>0.0)
      {
         _state=_norm+1.0;
      }
      else if(NewChange<0.0)
      {
         _state=1.0-_norm;
      }
      
      return(_state);
   }
//+------------------------------------------------------------------+
//| "Voting" that price will grow.                                   |
//+------------------------------------------------------------------+
int CSignalMC::LongCondition(void)
  {
   
   int result=0;
   int idx   =StartIndex();
   double _mc=ProcessMC(idx);
   
   if(_mc>0.0)
     {
      result=int(round(100.0*_mc));
     }
     
//--- return the result
   return(fmin(100,result));
  }
//+------------------------------------------------------------------+
//| "Voting" that price will fall.                                   |
//+------------------------------------------------------------------+
int CSignalMC::ShortCondition(void)
  {
   int result  =0;
   int idx     =StartIndex();
   double _mc=ProcessMC(idx);
   
   if(_mc<0.0)
     {
      result=int(round(-100.0*_mc));
     }
//--- return the result
   return(fmin(100,result));
  }
//+------------------------------------------------------------------+
